#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Script containing tests for the solventxmlreader.

@author: mark
"""

import logging
import unittest
from lxml import etree
import solventmapcreator.io.solventxmlreader as solventxmlreader

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

class SolventXMLReaderTestCase(unittest.TestCase):
    """Test case for solventxmlreader script tests.
    """
    def setUp(self):
        """Set up for tests.
        """
        self.solvent_list_xml = solventxmlreader.read_element_tree("resources/watersolvent.xml")
        self.water_solvent_element = self.solvent_list_xml.xpath("phase:Solvent", namespaces=solventxmlreader.PHASE_NAMESPACE_DICT)[0]
        self.water_molecule_element = self.water_solvent_element.xpath("phase:Molecule", namespaces=solventxmlreader.PHASE_NAMESPACE_DICT)[0]
        self.maxDiff=None
    def tearDown(self):
        """Tear down after tests.
        """
        del self.solvent_list_xml
        del self.water_solvent_element
        del self.water_molecule_element
    def test_get_fractional_occupancy_for_solvent_from_file(self):
        """Test to see if expected value is returned.
        """
        expected_value = 55.35 * 4.0 /300.0
        actual_value = solventxmlreader.get_fractional_occupancy_for_solvent_from_file("resources/watersolvent.xml", "water")
        self.assertAlmostEqual(expected_value, actual_value)
    def test_get_fractional_occupancy_for_solvent(self):
        """Test to see if expected fractional occupancy is returned.
        """
        expected_value = 55.35 * 4.0 /300.0
        actual_value = solventxmlreader.get_fractional_occupancy_for_solvent(self.solvent_list_xml, "water")
        self.assertAlmostEqual(expected_value, actual_value)
    def test_calculate_fractional_occupancy_for_solvent(self):
        """Test to see if expected value is returned for the fractional
        occupancy.
        """
        expected_value = 55.35 * 4.0 /300.0
        actual_value = solventxmlreader.calculate_fractional_occupancy_for_solvent(self.water_solvent_element)
        self.assertAlmostEqual(expected_value, actual_value)
    def test_calculate_fractional_occupancy_contribution(self):
        """Test to see if expected contribution to fractional occupancy is returned.
        """
        expected_value = 55.35* 4.0 / 300.0
        actual_value = solventxmlreader.calculate_fractional_occupancy_contribution(self.water_molecule_element)
        self.assertAlmostEqual(expected_value, actual_value)
    def test_get_concentration_value(self):
        """Test to see if expected concentration is returned
        """
        expected_value = 55.3500
        actual_value = solventxmlreader.get_concentration_value(self.water_molecule_element)
        self.assertAlmostEqual(expected_value, actual_value)
    def test_get_number_of_ssips_in_molecule(self):
        """Test to see if expected number of SSIPs is returned.
        """
        expected_value = 4
        actual_value = solventxmlreader.get_number_of_ssips_in_molecule(self.water_molecule_element)
        self.assertEqual(expected_value, actual_value)
    def test_get_solvent_element_by_id(self):
        """Test to see if expected solvent element is returned.
        """
        expected_xml = ('<phase:Solvent xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSchema" xmlns:cml="http://www.xml-cml.org/schema" xmlns:ssip="http://www-hunter.ch.cam.ac.uk/SSIP" phase:solventID="water" phase:solven' +
                        'tName="water" phase:numberOfMolecules="1">\n' + 
                        '    <phase:Molecule phase:stdInChIKey="XLYOFNOQVPJJNP-UHFFFAOYSA-N">\n' +
                        '      <ssip:SSIPList>\n' +
                        '\t\t\t<ssip:SurfaceInformation numberOfPoints="1000">\n' +
                        '\t\t\t\t<ssip:IsosurfaceDensity>0.002</ssip:IsosurfaceDensity>\n' +
                        '\t\t\t\t<ssip:TotalSurfaceArea>10.0</ssip:TotalSurfaceArea>\n' +
                        '\t\t\t\t<ssip:NegativeSurfaceArea>5.0</ssip:NegativeSurfaceArea>\n' +
                        '\t\t\t\t<ssip:PositiveSurfaceArea>5.0</ssip:PositiveSurfaceArea>\n' +
                        '\t\t\t</ssip:SurfaceInformation>\n' +
                        '\t\t\t<ssip:SSIP x3="1.0" y3="0.0" z3="0.0" nearestAtomID="a1">2.8</ssip:SSIP>\n' +
                        '\t\t\t<ssip:SSIP x3="1.0" y3="1.0" z3="1.0" nearestAtomID="a1">2.8</ssip:SSIP>\n' +
                        '\t\t\t<ssip:SSIP x3="0.0" y3="1.0" z3="1.0" nearestAtomID="a1">-4.5</ssip:SSIP>\n' +
                        '\t\t\t<ssip:SSIP x3="0.0" y3="1.0" z3="0.0" nearestAtomID="a1">-4.5</ssip:SSIP>\n' +
                        '\t\t</ssip:SSIPList>\n' +
                        '      <phase:Concentration phase:units="MOLAR">55.350000</phase:Concentration>\n' +
                        '    </phase:Molecule>\n' +
                        '  </phase:Solvent>\n')
        actual_xml = solventxmlreader.get_solvent_element_by_id(self.solvent_list_xml, "water")
        self.assertEqual(expected_xml, etree.tounicode(actual_xml))
    def test_create_solvent_xpath_expression(self):
        """Test to seeif expected xpath expression is created.
        """
        expected_expression = 'phase:Solvent[@phase:solventID="water"]'
        actual_expression = solventxmlreader.create_solvent_xpath_expression("water")
        self.assertEqual(expected_expression, actual_expression)
    def test_read_element_tree(self):
        """Test to see if expected file is read in.
        """
        expected_xml = ('<phase:SolventList xmlns:cml="http://www.xml-cml.org/sch' +
                        'ema" xmlns:phase="http://www-hunter.ch.cam.ac.uk/PhaseSc' +
                        'hema" xmlns:ssip="http://www-hunter.ch.cam.ac.uk/SSIP">' +
                        '\n  <phase:Solvent phase:solventID="water" phase:solven' +
                        'tName="water" phase:numberOfMolecules="1">\n' + 
                        '    <phase:Molecule phase:stdInChIKey="XLYOFNOQVPJJNP-UHFFFAOYSA-N">\n' +
                        '      <ssip:SSIPList>\n' +
                        '\t\t\t<ssip:SurfaceInformation numberOfPoints="1000">\n' +
                        '\t\t\t\t<ssip:IsosurfaceDensity>0.002</ssip:IsosurfaceDensity>\n' +
                        '\t\t\t\t<ssip:TotalSurfaceArea>10.0</ssip:TotalSurfaceArea>\n' +
                        '\t\t\t\t<ssip:NegativeSurfaceArea>5.0</ssip:NegativeSurfaceArea>\n' +
                        '\t\t\t\t<ssip:PositiveSurfaceArea>5.0</ssip:PositiveSurfaceArea>\n' +
                        '\t\t\t</ssip:SurfaceInformation>\n' +
                        '\t\t\t<ssip:SSIP x3="1.0" y3="0.0" z3="0.0" nearestAtomID="a1">2.8</ssip:SSIP>\n' +
                        '\t\t\t<ssip:SSIP x3="1.0" y3="1.0" z3="1.0" nearestAtomID="a1">2.8</ssip:SSIP>\n' +
                        '\t\t\t<ssip:SSIP x3="0.0" y3="1.0" z3="1.0" nearestAtomID="a1">-4.5</ssip:SSIP>\n' +
                        '\t\t\t<ssip:SSIP x3="0.0" y3="1.0" z3="0.0" nearestAtomID="a1">-4.5</ssip:SSIP>\n' +
                        '\t\t</ssip:SSIPList>\n' +
                        '      <phase:Concentration phase:units="MOLAR">55.350000</phase:Concentration>\n' +
                        '    </phase:Molecule>\n' +
                        '  </phase:Solvent>\n' +
                        '</phase:SolventList>')
        actual_xml = solventxmlreader.read_element_tree("resources/watersolvent.xml")
        self.assertEqual(expected_xml, etree.tounicode(actual_xml))
