#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Script for tests of the polynomial comparison.

@author: mark
"""

import logging
import unittest
import numpy as np
import solventmapcreator.polynomialanalysis.polynomialcomparison as polynomialcomparison

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

class PolynomialComparisonTestCase(unittest.TestCase):
    """Test case for the polynomialcomparison script.
    """
    def setUp(self):
        """Set up before tests
        """
        self.poly_data_by_solvent_id = {"poly_fit":{2:{"coefficients":np.array([0.0, 
                                                                                             0.0, 1.0]),
                                                                "RMSE":0.0254303,
                                                                "order":2,
                                                                "covar":0.0008143}},
                                        "poly_fit2":{2:{"coefficients":np.array([1.0, 
                                                                                          0.0, 1.0]),
                                                                 "RMSE":0.0354303,
                                                                 "order":2,
                                                                 "covar":0.0008143}},
                                        "poly_fit3":{2:{"coefficients":np.array([0.0, 
                                                                                          0.0, 2.0]),
                                                                 "RMSE":0.0354303,
                                                                 "order":2,
                                                                 "covar":0.0008143}}}
        self.x_values = np.array([-2.0, -1.0, 0.0, 1.0, 2.0])
        self.poly_values_dict = {"poly_fit":np.array([4.0, 1.0, 0.0, 1.0, 4.0]),
                                 "poly_fit2":np.array([5.0, 2.0, 1.0, 2.0, 5.0]), 
                                 "poly_fit3":np.array([8.0, 2.0, 0.0, 2.0, 8.0])}
    def tearDown(self):
        """Tear down after tests.
        """
        del self.poly_data_by_solvent_id
        del self.x_values
        del self.poly_values_dict
    def test_generate_similarity_matrix_plot_input(self):
        """Test to see if expected dictionary is created.
        """
        expected_dict = {"labels":["poly_fit", "poly_fit2", "poly_fit3"],
                         "value_matrix":np.array([[0.0, 1.0, 4.0],
                                                  [1.0, 0.0, 3.0],
                                                  [4.0, 3.0, 0.0]])}
        input_matrix = np.array([[0.0, 1.0, 4.0],
                                 [1.0, 0.0, 3.0],
                                 [4.0, 3.0, 0.0]])
        id_list = ["poly_fit", "poly_fit2", "poly_fit3"]
        actual_dict = polynomialcomparison.generate_similarity_matrix_plot_input(id_list, input_matrix)
        self.assertEqual(2, len(actual_dict.keys()))
        self.assertListEqual(sorted(expected_dict.keys()), sorted(actual_dict.keys()))
        self.assertListEqual(expected_dict["labels"], actual_dict["labels"])
        np.testing.assert_array_almost_equal(expected_dict["value_matrix"], actual_dict["value_matrix"])
    def test_calculate_hausdorff_distance_matrix_from_poly_coeff(self):
        """Test tp see if expected matrix is produced.
        """
        expected_matrix = np.array([[0.0, 1.0, 4.0],
                                    [1.0, 0.0, 3.0],
                                    [4.0, 3.0, 0.0]])
        actual_matrix = polynomialcomparison.calculate_hausdorff_distance_matrix_from_poly_coeff(self.x_values,
                                                                                                 2, self.poly_data_by_solvent_id)
        np.testing.assert_array_almost_equal(expected_matrix, actual_matrix)
    def test_calculate_hausdorff_distance_matrix(self):
        """Test to see if expected matrix is produced.
        """
        expected_matrix = np.array([[0.0, 1.0, 4.0],
                                    [1.0, 0.0, 3.0],
                                    [4.0, 3.0, 0.0]])
        actual_matrix = polynomialcomparison.calculate_hausdorff_distance_matrix(self.x_values,
                                                                                 self.poly_values_dict)
        np.testing.assert_array_almost_equal(expected_matrix, actual_matrix)
    def test_calculate_rmsd_matrix_from_poly_coeff(self):
        """Test to see if expected matrix is generated.
        """
        expected_matrix = np.array([[0.0, 1.0, 2.607681],
                                    [1.0, 0.0, 1.949359],
                                    [2.607681, 1.949359, 0.0]])
        actual_matrix = polynomialcomparison.calculate_rmsd_matrix_from_poly_coeff(self.x_values, 2, self.poly_data_by_solvent_id)
        np.testing.assert_array_almost_equal(expected_matrix, actual_matrix)
    def test_calculate_rmsd_matrix(self):
        """Test to see if expected RMSD matrix is produced.
        """
        expected_matrix = np.array([[0.0, 1.0, 2.607681],
                                    [1.0, 0.0, 1.949359],
                                    [2.607681, 1.949359, 0.0]])
        actual_matrix = polynomialcomparison.calculate_rmsd_matrix(self.poly_values_dict)
        np.testing.assert_array_almost_equal(expected_matrix, actual_matrix)
    def test_get_sorted_solvent_id_list(self):
        """Test to see if expected list is returned.
        """
        expected_list = ["poly_fit", "poly_fit2", "poly_fit3"]
        actual_list = polynomialcomparison.get_sorted_solvent_id_list(self.poly_values_dict)
        self.assertListEqual(expected_list, actual_list)
    def test_calculate_hausdorff_distance(self):
        """Test to see if expected Hausdorff distance is calculated.
        """
        expected_hausdorff_distance = 1.0
        actual_hausdorff_distance = polynomialcomparison.calculate_hausdorff_distance(self.x_values, np.array([4.0, 1.0, 0.0, 1.0, 4.0]),np.array([5.0, 2.0, 1.0, 2.0, 5.0]))
        self.assertAlmostEqual(expected_hausdorff_distance, actual_hausdorff_distance)
    def test_calculate_rmsd_between_curves(self):
        """test to see if expected RMSD is calculated.
        """
        expected_rmsd = 1.0
        actual_rmsd = polynomialcomparison.calculate_rmsd_between_curves(np.array([4.0, 1.0, 0.0, 1.0, 4.0]),np.array([5.0, 2.0, 1.0, 2.0, 5.0]))
        self.assertAlmostEqual(expected_rmsd, actual_rmsd)
    def test_create_array_for_hausdorff_analysis(self):
        """Test to see if expected array is produced.
        """
        expected_array = np.array([[-2.0, 4.0], [-1.0, 1.0], [0.0, 0.0], [1.0, 1.0], [2.0, 4.0]])
        actual_array = polynomialcomparison.create_array_for_hausdorff_analysis(self.x_values, np.array([4.0, 1.0, 0.0, 1.0, 4.0]))
        np.testing.assert_array_almost_equal(expected_array, actual_array)
    def test_calculate_polynomial_values_all_solvents(self):
        """Test to see if expected dictionary is produced.
        """
        expected_dict = {"poly_fit":np.array([4.0, 1.0, 0.0, 1.0, 4.0]),
                         "poly_fit2":np.array([5.0, 2.0, 1.0, 2.0, 5.0]), 
                         "poly_fit3":np.array([8.0, 2.0, 0.0, 2.0, 8.0])}
        actual_dict = polynomialcomparison.calculate_polynomial_values_all_solvents(self.x_values, 2, self.poly_data_by_solvent_id)
        self.assertListEqual(sorted(expected_dict.keys()), sorted(actual_dict.keys()))
        for key in expected_dict.keys():
            np.testing.assert_array_almost_equal(expected_dict[key], actual_dict[key])
    def test_calculate_polynomial_values_from_dict(self):
        """Test to see if expected array is produced.
        """
        expected_array = np.array([4.0, 1.0, 0.0, 1.0, 4.0])
        actual_array = polynomialcomparison.calculate_polynomial_values_from_dict(self.x_values, self.poly_data_by_solvent_id["poly_fit"], 2)
        np.testing.assert_array_almost_equal(expected_array, actual_array)
    def test_get_polynomial_coefficients(self):
        """Test to see if expected coefficients are returned.
        """
        expected_coefficients = np.array([0.0, 0.0, 1.0])
        actual_coefficients = polynomialcomparison.get_polynomial_coefficients(self.poly_data_by_solvent_id["poly_fit"], 2)
        np.testing.assert_array_almost_equal(expected_coefficients, actual_coefficients)
    def test_calculate_polynomial_values(self):
        """Test to see if expected values are returned.
        """
        expected_values = np.array([4.0, 1.0, 0.0, 1.0, 4.0])
        poly_coefficients = self.poly_data_by_solvent_id["poly_fit"][2]["coefficients"]
        actual_values = polynomialcomparison.calculate_polynomial_values(self.x_values, poly_coefficients)
        np.testing.assert_array_almost_equal(expected_values, actual_values)
